@@ -75,7 +75,7 @@ class IPTables(Collector):
 			try:
 				hash_old, metrics_old, warnings = self._table_hash[v]
 				if metrics is not metrics_old: raise KeyError
-			except KeyError: hash_old = None
+			except KeyError: hash_old, warnings = None, dict()
 			hash_new = hashes[v]
 
 			# iptables-save invocation and output processing loop
@@ -88,19 +88,26 @@ class IPTables(Collector):
 				counts, append, chain, rule = line.split(None, 3)
 				assert append == '-A'
 
-				chain_counts[chain] += 1 # iptables rules are 1-indexed
-				hash_new[chain].append(rule) # but py lists are 0-indexed
-				try: metric = metrics.table[table, chain, chain_counts[chain]]
+				rule_key = table, chain
+				chain_counts[rule_key] += 1 # iptables rules are 1-indexed
+				chain_count = chain_counts[rule_key]
+				# log.debug('{}, Rule: {}'.format([table, chain, chain_count], rule))
+				hash_new[rule_key].append(rule) # but py lists are 0-indexed
+				try: metric = metrics.table[table, chain, chain_count]
 				except KeyError: continue # no point checking rules w/o metrics attached
+				# log.debug('Metric: {} ({}), rule: {}'.format(
+				# 	metric, [table, chain, chain_count], rule ))
 
 				# Check for changed rules
-				if hash_old and hash_old[chain][chain_counts[chain]-1] != rule:
-					if chain_counts[chain] not in warnings:
+				try: rule_chk = hash_old and hash_old[rule_key][chain_count - 1]
+				except (KeyError, IndexError): rule_chk = None
+				if hash_old and rule_chk != rule:
+					if chain_count not in warnings:
 						log.warn(
 							( 'Detected changed netfilter rule (chain: {}, pos: {})'
 								' without corresponding rule_metrics file update: {}' )\
-							.format(chain, chain_counts[chain], ' '.join(rule)) )
-						warnings[chain_counts[chain]] = True
+							.format(chain, chain_count, rule) )
+						warnings[chain_count] = True
 					if self.conf.discard_changed_rules: continue
 
 				counts = map(int, counts.strip('[]').split(':', 1))
@@ -112,8 +119,8 @@ class IPTables(Collector):
 
 			# Detect if there are any changes in the table,
 			#  possibly messing the metrics, even if corresponding rules are the same
-			hash_new = dict( (chain, tuple(rules))
-				for chain, rules in hash_new.viewitems() )
+			hash_new = dict( (rule_key, tuple(rules))
+				for rule_key, rules in hash_new.viewitems() )
 			if hash_old\
 					and frozenset(hash_old.viewitems()) != frozenset(hash_new.viewitems()):
 				log.warn('Detected iptables changes without changes to rule_metrics file')
